@@ -39,7 +39,7 @@ sub repl_backtrace
 # return value from die
 sub repl_return
 {
-    (1, Sepia::repl_eval(@_));
+    (1, $Sepia::REPL{eval}->(@_));
 }
 
 sub repl_lsbreak
@@ -60,27 +60,63 @@ sub repl_lsbreak
 sub eval_in_env
 {
     my ($expr, $env) = @_;
-    local $::SEPIA_ENV = $env;
+    local $Sepia::ENV = $env;
     my $str = '';
     for (keys %$env) {
         next unless /^([\$\@%])(.+)/;
-        $str .= "local *$2 = \$::SEPIA_ENV->{'$_'}; ";
+        $str .= "local *$2 = \$Sepia::ENV->{'$_'}; ";
     }
     eval "do { no strict; $str $expr }";
 }
 
-## XXX: this is a better approach (the local business above is vile),
+sub tie_class
+{
+    my $sig = substr shift, 0, 1;
+    return $sig eq '$' ? 'Tie::StdScalar'
+        : $sig eq '@' ? 'Tie::StdArray'
+            : $sig eq '%' ? 'Tie::StdHash'
+                : die "Sorry, can't tie $sig\n";
+}
+
+# {
+#     require Tie::Array;
+#     require Tie::Hash;
+#     require Tie::Scalar;
+#     package Sepia::Array;
+#     our @ISA = qw(Tie::StdArray);
+#     sub TIEARRAY { bless $_[1], $_[0] }
+#     package Sepia::Hash;
+#     our @ISA = qw(Tie::StdHash);
+#     sub TIEHASH { bless $_[1], $_[0] }
+#     package Sepia::Scalar;
+#     our @ISA = qw(Tie::StdScalar);
+#     sub TIESCALAR { bless $_[1], $_[0] }
+# }
+
+# sub eval_in_env3
+# {
+#     my ($expr, $env) = @_;
+#     my @vars = grep /^([\$\@%])(.+)/, keys %$env;
+#     my $body = 'sub { my ('.join(',', @vars).');';
+#     for my $i (0..$#vars) {
+#         $body .= "tie $vars[$i], ".tie_class($vars[$i]).', $_['.$i.'];';
+#     }
+#     $body .= "$expr }";
+#     print STDERR "---\n$body\n---\n";
+#     $body = eval $body;
+#     $@ || $body->(@{$env}{@vars});
+# }
+
+## XXX: this is a better approach (the local/tie business is vile),
 ## but it segfaults and I'm not sure why.
 sub eval_in_env2
 {
-    my ($expr, $lev) = @_;
-    my $env = peek_my(2+$lev);
-    $lev += 4;
-    local $::SEPIA_ENV = $env;
+    my ($expr, $env, $fn) = @_;
+    local $Sepia::ENV = $env;
     my @vars = grep /^([\$\@%])(.+)/, keys %$env;
     my $body = 'sub { my ('.join(',', @vars).');';
     for (@vars) {
-        $body .= "Devel::LexAlias::lexalias($lev, '$_', \\$_);"
+        $body .= "Devel::LexAlias::lexalias(\$Sepia::ENV, '$_', \\$_);"
     }
     $body .= "$expr }";
     print STDERR "---\n$body\n---\n";
@@ -93,15 +129,22 @@ sub repl_upeval
 {
     my $exp = shift;
     # my ($lev, $exp) = $_[0] =~ /^\s*(\d+)\s+(.*)/;
-    print " <= $exp\n";
+    # print " <= $exp\n";
     # (0, eval_in_env2($exp, $level));
-    (0, eval_in_env($exp, peek_my(0+$level)));
+    print "=> ", eval_in_env($exp, peek_my(4+$level)), "\n";
+    # (0, eval_in_env3($exp, peek_my(4 + $level)));
+    0;
 }
 
 # inspect lexicals at level N, or current level
 sub repl_inspect
 {
-    my $i = shift || $level;
+    my $i = shift;
+    if ($i =~ /\d/) {
+        $i = 0+$i;
+    } else {
+        $i = $level + 3;
+    }
     my $sub = (caller $i)[3];
     if ($sub) {
         my $h = peek_my($i+1);
@@ -129,16 +172,26 @@ sub debug
     }
 }
 
+sub breakpoint_file
+{
+    my ($file) = @_;
+    return \%{$main::{"_<$file"}} if exists $main::{"_<$file"};
+    if ($file !~ /^\//) {
+        ($file) = grep /^_<.*\/\Q$file\E$/, keys %main::;
+        return \%{$main::{$file}} if $file;
+    }
+    return undef;
+}
+
 sub breakpoint
 {
     my ($file, $line, $cond) = @_;
-    if (!defined $main::{"_<$file"} && $file !~ /^\//) {
-        ($file) = grep /^_<.*\/\Q$file\E$/, keys %main::;
-        return unless $file;
-        $file =~ s/^_<//;
+    my $h = breakpoint_file $file;
+    if (defined $h) {
+        $h->{$line} = $cond || 1;
+        return "$file\:$line $h->{$line}";
     }
-    $main::{"_<$file"}{$line} = $cond || 1;
-    $file;
+    return undef;
 }
 
 sub repl_break
@@ -146,7 +199,7 @@ sub repl_break
     my $arg = shift;
     $arg =~ s/^\s+//;
     $arg =~ s/\s+$//;
-    my ($f, $l, $cond) = $arg =~ /^(.+?):(\d+)\s*(.*)/, $arg;
+    my ($f, $l, $cond) = $arg =~ /^(.+?):(\d+)\s*(.*)/;
     $cond ||= 1;
     $f ||= $file;
     $l ||= $line;
@@ -178,7 +231,8 @@ sub repl_delete
     my ($f, $l) = split /:/, shift;
     $f ||= $file;
     $l ||= $line;
-    delete $main::{"_<$f"}{$l};
+    my $h = breakpoint_file $f;
+    delete $h->{$l} if defined $h;
     0
 }
 
@@ -197,7 +251,7 @@ my %parent_doc = (
         'delete             Delete current breakpoint.',
     debug =>
         'debug [0|1]        Enable or disable debugging.',
-    lsbream =>
+    lsbreak =>
         'lsbreak            List breakpoints.',
 );
 
@@ -250,6 +304,7 @@ my %REPL = (
     eval => \&repl_upeval,
     return => \&repl_return,
     lsbreak => \&repl_lsbreak,
+    eval => \&repl_upeval,      # DANGER!
 );
 
 my %REPL_DOC = (
@@ -282,8 +337,6 @@ my %REPL_DOC = (
 sub repl
 {
     show_location;
-    # print STDERR $main::{'_<'.$file}[$line]
-    #     if defined $main::{'_<'.$file}[$line];
 
     local %Sepia::REPL = (%Sepia::REPL, %REPL, @_);
     local %Sepia::REPL_DOC = (%Sepia::REPL_DOC, %REPL_DOC);
@@ -327,8 +380,8 @@ sub die
         my $trace = $DB::trace;
         $DB::trace = 1;
         repl(
-            die => sub { local $STOPDIE=0; die @dieargs },
-            quit => sub { local $STOPDIE=0; die @dieargs });
+            die => sub { local $STOPDIE=0; CORE::die @dieargs },
+            quit => sub { local $STOPDIE=0; CORE::die @dieargs });
         $DB::trace = $trace;
     } else {
         CORE::die(Carp::shortmess @_);
@@ -346,8 +399,8 @@ sub warn
         local ($pack, $file, $line, $sub) = caller($level);
         print "@_\n\tin $sub\nWarned $MSG\n";
         repl(
-            warn => sub { local $STOPWARN=0; warn @dieargs },
-            quit => sub { local $STOPWARN=0; warn @dieargs });
+            warn => sub { local $STOPWARN=0; CORE::warn @dieargs },
+            quit => sub { local $STOPWARN=0; CORE::warn @dieargs });
         $DB::trace = $trace;
     } else {
         ## Avoid showing up in location information.
